import std.cstream;

import util;
import error;

class CommandLine
{
public:
	this(in string[] args)
	{
		m_errlog = new ErrorLog();
		
		const int arglen = args.length;
		int index = 1;
		wstring[] wargs = new wstring[arglen];
		
		// コマンドライン引数をUTF-16に変換
		foreach(i, arg ; args)
			wargs[i] = fromMBS(arg);
		
		// オプション解析
		for(; index < arglen; index++)
		{
			wstring arg = wargs[index];
			if(arg.length > 0 && arg[0] != '-')
			{
				break;
			}
			parseOption(arg);
		}
		
		// 残りは書式なので全て連結
		wstring format;
		for(; index < arglen; index++)
		{
			format ~= wargs[index] ~ (index != arglen - 1 ? " "w : null);
		}
		
		if(format.length != 0)
			m_format = format;
		else
			m_format = "yyyy m/d(DDD) h:nn:ss"w;
		
		if(!m_errlog.isEmpty() )
			throw new LoggedException(m_errlog);
	}
	
	// 個別のオプションを解析
	void parseOption(wstring opt)
	{
		wstring lopt = toLower(opt);
		
		if(opt.length < 2)
		{
			m_errlog.add("option error : 空のオプションを検出"w);
			return;
		}
		
		switch(lopt[1] )
		{
		case 'l': // ロケール
			m_locale = opt[2..$];
			break;
		case 'b': // 年号のベース
			try
			{
				m_base_year = toInteger(opt[2..$] );
			}
			catch
			{
				m_errlog.add("option error : 年号のベースが不正 "w ~ opt);
			}
			break;
		case 'f': // 桁埋め文字
			if(opt.length == 3)
			{
				m_fill_char = lopt[2];
			}
			else
			{
				m_errlog.add("option error : 桁埋め文字の指定が不正 "w ~ opt);
			}
			break;
		case 'm':
			if(opt.length == 2)
			{
				m_use_meridiem = true;
			}
			else
			{
				m_errlog.add("option error : 午前/午後の表示指定が不正 "w ~ opt);
			}
			break;
		default:
			m_errlog.add("option error : 不明なオプション "w ~ opt);
		}
	}
	
	wstring locale()
	{
		return m_locale;
	}
	
	wstring format()
	{
		return m_format;
	}
	
	int base_year()
	{
		return m_base_year;
	}
	
	wchar fill_char()
	{
		return m_fill_char;
	}
	
	bool use_meridiem()
	{
		return m_use_meridiem;
	}
	
private:
	wstring  m_locale;
	wstring  m_format;
	int      m_base_year;
	wchar    m_fill_char;
	bool     m_use_meridiem;
	ErrorLog m_errlog;
}
