import std.date;
import std.utf;
import std.cstream;

import util;
import error;

class DateStringBuilder
{
public:
	d_time time()
	{
		return m_time;
	}
	
	d_time time(d_time value)
	{
		m_year = YearFromTime(value);
		m_month = MonthFromTime(value);
		m_date = DateFromTime(value);
		m_day = WeekDay(value);
		m_hour = HourFromTime(value);
		m_minute = MinFromTime(value);
		m_second = SecFromTime(value);
		m_post_meridiem = (m_hour >= 12);
		return m_time = value;
	}
	
	wstring format()
	{
		return m_format;
	}
	
	wstring format(wstring value)
	{
		return m_format = value;
	}
	
	int base_year()
	{
		return m_base_year;
	}
	
	int base_year(int value)
	{
		return m_base_year = value;
	}
	
	wchar fill_char()
	{
		return m_fill_char;
	}
	
	wchar fill_char(wchar value)
	{
		if(value != wchar.init)
			m_fill_char = value;
		else
			m_fill_char = '0';
		
		return m_fill_char;
	}
	
	// 午前or午後の表示を使用するか
	bool use_meridiem()
	{
		return m_use_meridiem;
	}
	
	bool use_meridiem(bool value)
	{
		return m_use_meridiem = value;
	}
	
	int year()
	{
		return m_year;
	}
	
	int month() // 注) 1月が0、2月が1、...、12月が11
	{
		return m_month;
	}
	
	int date()
	{
		return m_date;
	}
	
	int day() // 日曜日が0、月曜日が1、...、土曜日が6
	{
		return m_day;
	}
	
	int hour() // 常に24時間制で返す
	{
		return m_hour;
	}
	
	int minute()
	{
		return m_minute;
	}
	
	int second()
	{
		return m_second;
	}
	
	bool isPM()
	{
		return m_post_meridiem;
	}
	
	bool isAM()
	{
		return !m_post_meridiem;
	}
	
	wchar[] yearString(const int len, const bool initialcap, const bool othercap)
	{
		if(len != 1 && len != 2 && len != 4)
			throw new Exception("keyword error : 「y」の数が不正");
		
		return util.toWCharArray(m_year - m_base_year, len, m_fill_char);
	}
	
	wchar[] monthString(const int len, const bool initialcap, const bool othercap)
	{
		if(len < 1 && 4 < len)
			throw new Exception("keyword error : 「m」の数が不正");
		
		return util.toWCharArray(m_month + 1, len, m_fill_char);
	}
	
	wchar[] dateString(const int len, const bool initialcap, const bool othercap)
	{
		if(len != 1 && len != 2)
			throw new Exception("keyword error : 「d」の数が不正");
		
		return util.toWCharArray(m_date, len, m_fill_char);
	}
	
	abstract wchar[] dayString(const int len, const bool initialcap, const bool othercap);
	
	wchar[] hourString(const int len, const bool initialcap, const bool othercap)
	{
		if(len != 1 && len != 2)
			throw new Exception("keyword error : 「h」の数が不正");
		
		int hourtmp = m_hour;
		
		if(m_use_meridiem && m_post_meridiem)
		{
			hourtmp -= 12;
		}
		
		return util.toWCharArray(hourtmp, len, m_fill_char);
	}
	
	wchar[] minuteString(const int len, const bool initialcap, const bool othercap)
	{
		if(len != 1 && len != 2)
			throw new Exception("keyword error : 「n」の数が不正");
		
		return util.toWCharArray(m_minute, len, m_fill_char);
	}
	
	wchar[] secondString(const int len, const bool initialcap, const bool othercap)
	{
		if(len != 1 && len != 2)
			throw new Exception("keyword error : 「s」の数が不正");
		
		return util.toWCharArray(m_second, len, m_fill_char);
	}
	
	abstract wchar[] meridiemString(const int len, const bool initialcap, const bool othercap);

	wstring toWString()
	{
		ErrorLog errlog = new ErrorLog();
		
		wchar[] res;
		const int fmtlen = m_format.length;
		
		// バッファのメモリ空間を予約
		res.length = DEFAULT_BUFF_LEN;
		res.length = 0;
		
		int i = 0;
		while(i < fmtlen)
		{
			int step;
			try
			{
				wstring fmttmp = m_format[i..$];
				
				if(parseEscape(fmttmp, res, step) )
				{}
				else if(parseQuote(fmttmp, res, step) )
				{}
				else if(parseKeyword(fmttmp, res, step) )
				{}
				else
				{
					res ~= fmttmp[0..1];
					step = 1;
				}
			}
			catch(Exception exc)
			{
				res ~= "[err]"w;
				errlog.add(exc);
			}
			i += step;
		}
		
		if(!errlog.isEmpty() )
		{
			errlog.add(res.idup);
			throw new LoggedException(errlog);
		}
		
		return res.idup;
	}
	
private:
	// start   ... 書式の部分文字列
	// outbuff ... 出力はこのバッファに連結する
	// step    ... 書式を何文字解析したか
	bool parseEscape(in wstring start, ref wchar[] outbuff, out int step)
	{
		if(start[0] != '\\')
			return false;
		
		// 書式の最後が\の場合、何もしない
		if(start.length < 2)
		{
			step = 0;
			return true;
		}
		
		step = 2;
		switch(start[1] )
		{
		case 'r':
			outbuff ~= "\r";
			break;
		case 'n':
			outbuff ~= "\n";
			break;
		case 't':
			outbuff ~= "\t";
			break;
		case '`':
			outbuff ~= "`";
			break;
		case '"':
			outbuff ~= "\"";
			break;
		case '\\':
			outbuff ~= "\\";
			break;
		case 'x':
			wchar ch;
			try
			{
				ch = toWChar(start[2..6] );
			}
			catch
			{
				throw new Exception("escape error : \\xの書式エラー");
			}
			outbuff.length = outbuff.length + 1;
			outbuff[$ - 1] = ch;
			step = 6;
			break;
		default:
			throw new Exception("escape error : 未対応のエスケープ文字");
		}
		
		return true;
	}
	
	// 引用符の解析
	// 「"」と「`」を引用符として扱う
	// 引用符の中はエスケープ文字を無視する
	bool parseQuote(in wstring start, ref wchar[] outbuff, out int step)
	{
		if(start[0] != '"' && start[0] != '`')
			return false;
		
		const int len = start.length;
		
/*		// 書式の最後が引用符の場合、何もしない
		if(len < 2)
		{
			step = 0;
			return true;
		}
*/		
		// 終わりの引用符を探す内部関数
		void find_last_and_cat(wchar ch)
		{
			int i;
			for(i = 1; i < len; i++)
			{
				if(start[i] == ch)
				{
					break;
				}
			}
			if(i == len)
			{
				step = 1;
				throw new Exception("quote error : 引用符が閉じていない");
			}
			
			outbuff ~= start[1..i];
			step = i + 1;
		}
		
		switch(start[0] )
		{
		case '"':
			find_last_and_cat('"');
			return true;
		case '`':
			find_last_and_cat('`');
			return true;
		default:
			return false;
		}
	}
	
	bool parseKeyword(in wstring start, ref wchar[] outbuff, out int step)
	{
		bool initialcap, othercap;
		
		if(startAt(start, 'y', step, initialcap, othercap) )
			outbuff ~= yearString(step, initialcap, othercap);
		else if(startAt(start, 'm', step, initialcap, othercap) )
			outbuff ~= monthString(step, initialcap, othercap);
		else if(startAt(start, 'd', step, initialcap, othercap) )
		{
			if(step <= 2)
			{
				outbuff ~= dateString(step, initialcap, othercap);
			}
			else
			{
				outbuff ~= dayString(step, initialcap, othercap);
			}
		}
		else if(startAt(start, 'h', step, initialcap, othercap) )
			outbuff ~= hourString(step, initialcap, othercap);
		else if(startAt(start, 'n', step, initialcap, othercap) )
			outbuff ~= minuteString(step, initialcap, othercap);
		else if(startAt(start, 's', step, initialcap, othercap) )
			outbuff ~= secondString(step, initialcap, othercap);
		else if(startAt(start, 't', step, initialcap, othercap) )
		{
			if(!m_use_meridiem)
			{
				throw new Exception("keyword error : 「tt」を使用する場合はオプション指定が必要");
			}
			outbuff ~= meridiemString(step, initialcap, othercap);
		}
		else
			return false;
		
		return true;
	}
	
	// str        ... 判定する文字列
	// key        ... この文字で始まっているかを判定する, 必ず小文字で指定
	// len        ... chが何文字つながっているかを返す
	// initialcap ... 頭文字が大文字かを返す
	// othercap   ... 2文字目以降が大文字かを返す(全て大文字でtrue)
	bool startAt(const wstring str, const wchar key, out int len, out bool initialcap, out bool othercap)
	{
		int i;
		wchar keycap = key + 'A' - 'a';
		othercap = true;
		
		for(i = 0; i < str.length; i++)
		{
			wchar c = str[i];
			
			if( (c != key) && (c != keycap) )
			{
				break;
			}
			
			if(i == 0) // 頭文字が大文字かを判定
			{
				initialcap = (c == keycap);
			}
			else // 2文字目以降が大文字かを判定
			{
				othercap &= (c == keycap);
			}
		}
		
		len = i;
		return i != 0;
	}
	
	static const int DEFAULT_BUFF_LEN = 255;

	d_time  m_time;
	wstring m_format;
	int     m_yearbase;
	bool    m_post_meridiem;
	int     m_base_year;
	wchar   m_fill_char = '0';
	int     m_year;
	int     m_month;
	int     m_date;
	int     m_day;
	int     m_hour;
	int     m_minute;
	int     m_second;
	bool    m_use_meridiem;
}
