module test.datestringbuilder_test;

import std.cstream;
import std.stdio;
import std.utf;
import std.string;
import std.date;

import util;
import datestringbuilderjpn;
import datestringbuildereng;
import datestringbuilderjpnk;
import datestringbuilder;
import test.test_out;

void datestringbuilder_test_start()
{
	DateStringBuilder[3] dsbs;
	dsbs[0] = new DateStringBuilderEng();
	dsbs[0].fill_char = '*';
	dsbs[1] = new DateStringBuilderJpn();
	dsbs[1].fill_char = '*';
	dsbs[2] = new DateStringBuilderJpnK();
	dsbs[2].fill_char = '※';

	test_quote(dsbs);
	test_second(dsbs);
	test_minute(dsbs);
	test_hour(dsbs);
	test_meridiem(dsbs);
	test_day(dsbs);
	test_date(dsbs);
	test_month(dsbs);
	test_year(dsbs);
	pickup(dsbs);
}

void test_quote(DateStringBuilder[] dsbs)
{
	write_log("=========================== 引用符テスト ==========================="w);
	
	wstring format = `\"\x4fe1\"`w;
	write_log("=========================(" ~ format ~ ")=========================" );
	set_param(dsbs, format, 2008, 9, 6,       7, 23, 0);
	
	format = `"\x4fe1"`w;
	write_log("=========================(" ~ format ~ ")=========================" );
	set_param(dsbs, format, 2008, 9, 6,       7, 23, 0);
	
	format = "here you are. `here you are.`"w;
	write_log("=========================(" ~ format ~ ")=========================" );
	set_param(dsbs, format, 2008, 9, 6,       7, 23, 0);
	
	format = "here you are. `here you are."w;
	write_log("=========================(" ~ format ~ ")=========================" );
	set_param(dsbs, format, 2008, 9, 6,       7, 23, 0);
	
	format = "0123456789`"w;
	write_log("=========================(" ~ format ~ ")=========================" );
	set_param(dsbs, format, 2008, 9, 6,       7, 23, 0);
}

void test_second(DateStringBuilder[] dsbs)
{
	write_log("=========================== 秒テスト ==========================="w);
	wstring[] format_list = ["s"w, "S"w, "ss"w, "Ss"w, "sS"w, "SS"w, "sss"w];
	wstring[] second_list = ["0"w, "9"w, "10"w, "19"w, "50"w, "59"w];
	
	foreach(format ; format_list)
	{
		write_log("=========================(" ~ format ~ ")=========================" );
		foreach(second ; second_list)
		{
			write_log("* 書式(" ~ format ~ ") 2008 10/6(???) 7:23:"w ~ second);
			set_param(dsbs, format, 2008, 9, 6,       7, 23, toInteger(second));
		}
	}
}

void test_minute(DateStringBuilder[] dsbs)
{
	write_log("=========================== 分テスト ==========================="w);
	wstring[] format_list = ["n"w, "N"w, "nn"w, "Nn"w, "nN"w, "NN"w, "nnn"w];
	wstring[] minute_list = ["0"w, "9"w, "10"w, "19"w, "50"w, "59"w];
	
	foreach(format ; format_list)
	{
		write_log("=========================(" ~ format ~ ")=========================" );
		foreach(minute ; minute_list)
		{
			write_log("* 書式(" ~ format ~ ") 2008 10/6(???) 7:" ~ minute ~ ":36"w);
			set_param(dsbs, format, 2008, 9, 6,       7, toInteger(minute), 36);
		}
	}
}

void test_hour(DateStringBuilder[] dsbs)
{
	write_log("=========================== 時テスト ==========================="w);
	wstring[] format_list = ["h"w, "H"w, "hh"w, "Hh"w, "hH"w, "HH"w, "hhh"w];
	wstring[] hour_list = ["0"w, "9"w, "10"w, "11"w, "12"w, "20"w, "23"w];
	
	set_use_meridiem(dsbs, true);
	foreach(format ; format_list)
	{
		write_log("=========================(" ~ format ~ ")=========================" );
		foreach(hour ; hour_list)
		{
			write_log("* 書式(" ~ format ~ " t) 2008 10/6(???) " ~ hour ~ ":11:36"w);
			set_param(dsbs, format ~ " t"w, 2008, 9, 6,       toInteger(hour), 11, 36);
		}
	}
	
	set_use_meridiem(dsbs, false);
	foreach(format ; format_list)
	{
		write_log("=========================(" ~ format ~ ")=========================" );
		foreach(hour ; hour_list)
		{
			write_log("* 書式(" ~ format ~ ") 2008 10/6(???) " ~ hour ~ ":11:36"w);
			set_param(dsbs, format, 2008, 9, 6,       toInteger(hour), 11, 36);
		}
	}
}

void test_meridiem(DateStringBuilder[] dsbs)
{
	write_log("=========================== 午テスト ==========================="w);
	wstring[] format_list = ["t"w, "T"w, "tt"w, "Tt"w, "tT"w, "TT"w, "ttt"w];
	wstring[] hour_list = ["0"w, "11"w, "12"w, "23"w];
	
	set_use_meridiem(dsbs, true);
	
	foreach(format ; format_list)
	{
		write_log("=========================(" ~ format ~ ")=========================" );
		foreach(hour ; hour_list)
		{
			write_log("* 書式(" ~ format ~ ") 2008 10/6(???) " ~ hour ~ ":11:36"w);
			set_param(dsbs, format, 2008, 9, 6,       toInteger(hour), 11, 36);
		}
	}
}

void test_day(DateStringBuilder[] dsbs)
{
	write_log("=========================== 曜テスト ==========================="w);
	wstring[] format_list = ["ddd"w, "Ddd"w, "DDD"w, "dddd"w, "Dddd"w, "DDDD"w, "dDDD"w, "DddD"w, "ddddd"w, "d``Dddd"w];
	wstring[] day_list = ["1"w, "2"w, "3"w, "4"w, "5"w, "6"w, "7"w];
	
	foreach(format ; format_list)
	{
		write_log("=========================(" ~ format ~ ")=========================" );
		foreach(day ; day_list)
		{
			write_log("* 書式(" ~ format ~ ") 2008 10/" ~ day ~ "(???) 15:11:36"w);
			set_param(dsbs, format, 2008, 9, toInteger(day),       15, 11, 36);
		}
	}
}

void test_date(DateStringBuilder[] dsbs)
{
	write_log("=========================== 日テスト ==========================="w);
	wstring[] format_list = ["d"w, "D"w, "dd"w, "Dd"w, "DD"w];
	wstring[] date_list = ["1"w, "10"w, "31"w];
	
	foreach(format ; format_list)
	{
		write_log("=========================(" ~ format ~ ")=========================" );
		foreach(date ; date_list)
		{
			write_log("* 書式(" ~ format ~ ") 2008 10/" ~ date ~ "(???) 15:11:36"w);
			set_param(dsbs, format, 2008, 9, toInteger(date),       15, 11, 36);
		}
	}
}

void test_month(DateStringBuilder[] dsbs)
{
	write_log("=========================== 月テスト ==========================="w);
	wstring[] format_list = ["m"w, "M"w, "mm"w, "Mm"w, "MM"w, "mmm"w, "Mmm"w, "MMM"w, "mmmm"w, "Mmmm"w, "MMMM"w, "mMMM"w, "MmMm"w, "mmmmm"w];
	wstring[] month_list = ["0"w, "1"w, "2"w, "3"w, "4"w, "5"w, "6"w, "7"w, "8"w, "9"w, "10"w, "11"w];
	
	foreach(format ; format_list)
	{
		write_log("=========================(" ~ format ~ ")=========================" );
		foreach(month ; month_list)
		{
			write_log("* 書式(" ~ format ~ ") 2008 " ~ month ~ "/6(???) 15:11:36"w);
			set_param(dsbs, format, 2008, toInteger(month), 6,       15, 11, 36);
		}
	}
}

void test_year(DateStringBuilder[] dsbs)
{
	write_log("=========================== 年テスト ==========================="w);
	wstring[] format_list = ["y"w, "Y"w, "yy"w, "YY"w, "Yy"w, "yyy"w, "yyyy"w, "YYYY"w, "Yyyy"w, "YyYy"w, "yyyyy"w, "yYYY"w];
	wstring[] year_list = ["0"w, "1"w, "23"w, "456"w, "7890"w, "9999"w, "10000"w, "279988"w, "279989"w];
	
	foreach(format ; format_list)
	{
		write_log("=========================(" ~ format ~ ")=========================" );
		foreach(year ; year_list)
		{
			write_log("* 書式(" ~ format ~ ") " ~ year ~ " 11/6(???) 15:11:36"w);
			set_param(dsbs, format, toInteger(year), 10, 6,       15, 11, 36);
		}
	}
}

void pickup(DateStringBuilder[] dsbs)
{
	write_log("=========================== 書式ピックアップテスト ==========================="w);
	
	write_log("* デフォルト書式(yyyy m/d(DDD) h:nn:ss) 2008 11/6(THU) 15:11:36"w);
	set_param(dsbs, "yyyy m/d(DDD) h:nn:ss"w, 2008, 10, 6,       15, 11, 36);

	write_log("* ピックアップ書式1(Yyyy m/d(DDDD) h:n:s) 105 5/15(???) 9:6:6"w);
	set_param(dsbs, "Yyyy m/d(DDDD) h:n:s"w, 105, 4, 15,       9, 6, 6);

	write_log("* ピックアップ書式2(YYYY mm/dd(Dddd) hh:nn:ss) 3218 9/20(???) 13:31:17"w);
	set_param(dsbs, "YYYY mm/dd(Dddd) hh:nn:ss"w, 3218, 8, 20,       13, 31, 17);

	write_log("* ピックアップ書式3(平成y年m月d日(Ddd) h時nn分ss秒) 20 1/31(???) 21:0:20"w);
	set_param(dsbs, "平成y年m月d日(Ddd) h時n分s秒"w, 20, 0, 31,       21, 0, 20);

	write_log("* ピックアップ書式4(平成Y年M月D日(ddDd) HH時NN分SS秒) 1 4/7(???) 0:31:50"w);
	set_param(dsbs, "平成Y年M月D日(ddDd) HH時NN分SS秒"w, 1, 3, 7,       0, 31, 50);

	set_use_meridiem(dsbs, true);
	write_log("* ピックアップ書式5(Dddd, Mmm d, y H:NN:SS t) 908 3/16(???) 8:27:48"w);
	set_param(dsbs, "Dddd, Mmm d, y H:NN:SS t"w, 908, 2, 16,       8, 27, 48);

	write_log("* ピックアップ書式6(Mmm. d, y (Ddd.) TTHH:N:S) 2010 12/28(???) 19:48:9"w);
	set_param(dsbs, "Mmm. d, y (Ddd.) TTHH:N:S"w, 2010, 11, 28,       19, 48, 9);
	set_use_meridiem(dsbs, false);

	write_log("* ピックアップ書式7(Ddd, dd Mmm yyyy hh:nn:dd +0900) 1995 2/13(???) 5:7:39"w);
	set_param(dsbs, "Ddd, dd Mmm yyyy hh:nn:dd +0900"w, 1995, 1, 13,       5, 7, 39);

	write_log("* ピックアップ書式8(Ddd Mmm dd HH:NN:SS `JST` YYYY) 2000 7/29(???) 3:0:0"w);
	set_param(dsbs, "Ddd Mmm dd HH:NN:SS `JST` YYYY"w, 2000, 7, 29,       3, 0, 0);
}

void set_param(DateStringBuilder[] dsbs, wstring format, d_time y, d_time m, d_time d, d_time h, d_time n, d_time s)
{
	d_time t = MakeDate(MakeDay(y, m, d), MakeTime(h, n, s, 0) );
	dsbs[0].format = format;
	dsbs[1].format = format;
	dsbs[2].format = format;
	dsbs[0].time = t;
	dsbs[1].time = t;
	dsbs[2].time = t;
	try{
		write_log("    Eng  : " ~ dsbs[0].toWString() );
	}catch(Exception exc){write_log(exc);}
	try{
		write_log("    Jpn  : " ~ dsbs[1].toWString() );
	}catch(Exception exc){write_log(exc);}
	try{
		write_log("    JpnK : " ~ dsbs[2].toWString() );
	}catch(Exception exc){write_log(exc);}
}

void set_use_meridiem(DateStringBuilder[] dsbs, bool value)
{
	dsbs[0].use_meridiem = value;
	dsbs[1].use_meridiem = value;
	dsbs[2].use_meridiem = value;
}
