import std.cstream;
import std.string;
import win32.core;
import std.utf;

void setClipboard(in wstring str)
{
	BOOL chk;
	HANDLE chk_hdl;
	
	const int len = str.length;
	HANDLE hGlobal = GlobalAlloc(GHND, (len + 1) * wchar.sizeof);
	if(hGlobal is null)
		throw new Exception("util error : [setClipboard]メモリ確保失敗");
	scope(failure) GlobalFree(hGlobal);
	
	wchar* p = cast(wchar*)GlobalLock(hGlobal);
	if(p is null)
		throw new Exception("util error : [setClipboard]メモリロック失敗");

	p[0..len] = str[0..$];
	p[len] = '\0';
	GlobalUnlock(hGlobal);
	
	chk = OpenClipboard(null);
	if(chk == 0)
		throw new Exception("util error : [setClipboard]クリップボードのオープン失敗");
	
	chk = EmptyClipboard();
	if(chk == 0)
		throw new Exception("util error : [setClipboard]クリップボードの所有権取得失敗");
	
	chk_hdl = SetClipboardData(CF_UNICODETEXT, hGlobal);
	if(chk_hdl is null)
		throw new Exception("util error : [setClipboard]クリップボードへの書き込み失敗");
	
	CloseClipboard();
}

int toInteger(wstring str)
{
	if(str.length == 0)
		throw new Exception("util error : [toInteger]文字列が空");
	
	bool minus;
	int res;
	
	foreach(i, ch ; str)
	{
		if(ch < '0' || '9' < ch)
		{
			if(i == 0 && ch =='-')
			{
				minus = true;
				continue;
			}
			else
			{
				throw new Exception("util error : [toInteger]数字ではない");
			}
		}
		
		res *= 10;
		res += cast(int)(ch - '0');
	}
	
	return minus ? -res : res;
}

wchar[] toWCharArray(in int value, const int col, const wchar fill)
{
	const int buffsize = 10;
	
	// 9桁まで(buffsizeに依存)
	if(value < 0 || 1000000000 <= value)
		throw new Exception("util error : [toWCharArray]変換できる値は0～999999999");
	if(col <= 0 || 9 < col)
		throw new Exception("util error : [toWCharArray]変換できる値は9桁まで");
	
	wchar[] res = new wchar[buffsize];
	res[] = fill;
	int i = buffsize - 1;
	
	do
	{
		res[i] = '0' + (value % 10);
		
		value /= 10;
		i--;
	}
	while( (i >= 0) && (value != 0) );
	
	return res[ (col > 1 ? buffsize - col : i + 1)..buffsize];
}

// use0 ... 漢数字の「〇」を使用するかどうか。
// col  ... 桁数指定。use0がtrueの場合のみ使用。1未満で指定なし。
// fill ... 桁数指定があり、桁が足りないときにこの文字で埋める。
wchar[] toWCharArrayK(in int value, const bool use0, const int col, const wchar fill)
{
	// 4桁まで
	if(value < 0 || 10000 <= value)
		throw new Exception("util error : [toWCharArrayK]漢数字に変換できる値は0～9999");
	if(use0 && (col <= 0 || 4 < col) )
		throw new Exception("util error : [toWCharArrayK]漢数字に変換できる値は4桁まで");
	
	// 「〇」を使用せず、入力値が0の場合は即値を返す
	if(!use0 && value == 0)
		return "零"w.dup;
	
	// 4桁用のパラメータ
	const int buffsize = 10;
	const wstring columnchar = "十百千";
	
	wchar[] res = new wchar[buffsize];
	res[] = fill;
	int i = buffsize - 1;  // resの添字、右端からスタート
	int t = value % 10;    // 各位の数字
	int k = 0;             // 何の位か、columncharの添字
	
	do
	{
		if(use0 || t > 1 || (t == 1 && k == 0) )
		{
			switch(t)
			{
			case 0: res[i] = '〇'; break;
			case 1: res[i] = '一'; break;
			case 2: res[i] = '二'; break;
			case 3: res[i] = '三'; break;
			case 4: res[i] = '四'; break;
			case 5: res[i] = '五'; break;
			case 6: res[i] = '六'; break;
			case 7: res[i] = '七'; break;
			case 8: res[i] = '八'; break;
			case 9: res[i] = '九'; break;
			}
			i--;
		}
		
		value /= 10;
		t = value % 10;
		
		if(!use0 && t != 0)
		{
			res[i] = columnchar[k];
			i--;
		}
		k++;
	}
	while(value > 0);
	
	return res[ (use0 && col > 1 ? buffsize - col : i + 1)..buffsize];
}

wstring toLower(wstring str)
{
	wchar[] res = new wchar[str.length];
	
	foreach(i, ch ; str)
		res[i] = ('A' <= ch && ch <='Z') ? ch - 'A' + 'a' : ch;
	
	return res.idup;
}

// str        ... 入力文字列
// initialcap ... trueの場合、頭文字を大文字にする
// othercap   ... trueの場合、2文字目以降を大文字にする
wchar[] toUpper(wchar[] str, const bool initialcap, const bool othercap)
{
	if(initialcap)
		str[0] = toUpper(str[0] );

	if(othercap)
	{
		const int len = str.length;
		for(int i = 1; i < len; i++)
		{
			str[i] = toUpper(str[i] );
		}
	}
	
	return str;
}

wchar toUpper(const wchar ch)
{
	if('a' <= ch && ch <='z')
		return ch - 'a' + 'A';
	else
		return ch;
}

string toMBS(in string src)
{
	return toMBS(toUTF16(src) );
}

string toMBS(in wstring src)
{
	char[] dst;
	int len = 0;
	
	len = WideCharToMultiByte(
		0u,
		0u,
		cast(wchar*)src.ptr,
		src.length, // 0終端の場合-1でよい、D言語は0終端ではないので配列長を渡す。
		null,
		0, // 出力バッファのサイズ、0にすると必要サイズを返す
		null,
		null);
	dst.length = len;
	WideCharToMultiByte(
		0u,
		0u,
		cast(wchar*)src.ptr,
		src.length, // 0終端の場合-1でよい
		dst.ptr,
		len, // 出力バッファのサイズ
		null,
		null);
	return dst.idup;
}

wstring fromMBS(in string src)
{
	wchar[] dst;
	int len = 0;
	
	len = MultiByteToWideChar(
		0u,
		0u,
		cast(char*)src.ptr,
		src.length,
		null,
		0); // 出力バッファのサイズ、0にすると必要サイズを返す
	dst.length = len;
	MultiByteToWideChar(
		0u,
		0u,
		cast(char*)src.ptr,
		src.length,
		dst.ptr,
		len); // 出力バッファのサイズ
	
	return dst.idup;
}

// エスケープ文字「\x????」用。入力文字列を数値に変換して返す。
// 入力文字列は16進数4桁と仮定して処理。
// 呼び出し元で例外をcatchしているので桁のチェックは省略
wchar toWChar(in wstring str)
{
	wchar res;
	
	for(int i = 0; i < 4; i++) // 4桁固定
	{
		res *= 16;
		
		const wchar c = str[i];
		if('0' <= c && c <= '9')
		{
			res += c - '0';
		}
		else if('a' <= c && c <= 'f')
		{
			res += 10 + c - 'a';
		}
		else if('A' <= c && c <= 'F')
		{
			res += 10 + c - 'A';
		}
		else
		{
			throw new Exception("util error : [toWChar]入力文字列が16進数ではない");
		}
	}
	
	return res;
}
